const Util = (function () {

  const WHITE_FIRST = /^\s/
  const WHITE_LAST = /\s$/
  const WHITE_ALL = /^\s*$/
  const  DIGITS = /^[0-9]+$/
  const ALPHA = /^[a-zA-Z]+$/
  const ALPHANUM = /^[a-zA-Z0-9]+$/

  function is_white(str) {
    return WHITE_ALL.test(str);
  }

  function is_digits(str) {
    return DIGITS.test(str);
  }

  function is_alphanum(str) {
    return ALPHANUM.test(str); // Caution: Accepts leading digit
  }

  function trim_lead(s) {
    if (s.length>0) {
      while(WHITE_FIRST.test(s)) {
        s=s.slice(1); 
      }
    }
    return s
  }

  function trim_trail(s) {
    if (s.length>0) {
      while(WHITE_LAST.test(s)) {
        s=s.slice(0,s.length-1);
      }
    }
    return s
  }

  function trim(s) {
    return trim_lead(trim_trail(s))
  }

  function _split_dot (s) {
    const n=s.toString().indexOf('.')
    if (n>0) {
      const head = trim(s.slice(0, n))
      const tail = s.slice(n+1)
      return _split_index(head).concat(tail)
    } else {
      return _split_index(s)
    }
  }

  function _split_index(s) {
    const n0=s.toString().indexOf('[')
    if (n0>0) {
      const n1=s.indexOf(']')
      if (n1>n0 && n1===s.length-1) {
        const head = trim(s.slice(0, n0))
        const tail = s.slice(n0+1, n1)
        if (is_digits(tail)) {
          return [head, tail]
        } else {
          return [s]
        }
      } else {
        return [s]
      }
    } else {
      return [s]
    }
   
  }

  function split_jspath(s) {
    const arr=_split_dot(s)
    if (arr.length===2) {
      return [arr[0]].concat(split_jspath(arr[1]))
    } else {
      return arr
    }
  }

  function is_valid_number (s) {
    return !(is_white(s) || isNaN(s))
  }

  function is_valid_integer (s) {
    const RE = /^[-+0123456789]+$/
    return RE.test(s) && is_valid_number(s)
  }

  
  return {
    is_white,
    is_digits,
    is_alphanum,
    trim_lead,
    trim_trail,
    trim,
    split_jspath,
    is_valid_number,
    is_valid_integer,
  }
}());

module.exports=Util

